# -*- coding: utf-8 -*-
"""
Created on Sun May  4 21:09:01 2025
Calcula 
@author: hslopes & Gemini
"""
import cv2
import numpy as np
import pandas as pd
import os
from skimage.feature import local_binary_pattern

def calcula_HOG(imagens):
    """
    Calcula os Histogramas de Gradientes Orientados (HOG) para cada imagem.

    Args:
        imagens (dict): Um dicionário contendo os nomes dos arquivos
                       e as imagens carregadas.

    Returns:
        pandas.DataFrame: Um DataFrame onde a primeira coluna
                          contém os nomes das imagens e as colunas subsequentes
                          contêm os descritores HOG.
    """
    hog = cv2.HOGDescriptor()  # Inicializa o descritor HOG

    hog_vectors = []
    num_elementos_hog = 0

    for nome_arquivo, img in imagens.items():
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        # Calcula o descritor HOG
        hog_descriptor = hog.compute(gray)
        if hog_descriptor is not None and len(hog_descriptor) > 0:
            hog_vectors.append([nome_arquivo] + hog_descriptor.flatten().tolist())
            if not num_elementos_hog:
                num_elementos_hog = len(hog_descriptor.flatten())
        else:
            print(f"Nenhuma feature HOG detectada em: {nome_arquivo}")
            hog_vectors.append([nome_arquivo] + [0] * (num_elementos_hog if num_elementos_hog > 0 else 128))  # Mantendo 128 como valor padrão

    # Correção: Garante que num_elementos_hog tenha um valor válido mesmo se nenhuma feature for detectada
    if not num_elementos_hog:
        num_elementos_hog = 128  # Valor padrão

    colunas = ["nome_arquivo"] + [f"HOG_{i}" for i in range(num_elementos_hog)]
    df_hog = pd.DataFrame(hog_vectors, columns=colunas)
    print(f"Número de elementos do vetor HOG: {num_elementos_hog}")
    return df_hog


def calcula_LBP(imagens):
    """
    Calcula o Linear Binary Pattern (LBP) para cada imagem.

    Args:
        imagens (dict): Um dicionário contendo os nomes dos arquivos
                       e as imagens carregadas.

    Returns:
        pandas.DataFrame: Um DataFrame onde a primeira coluna
                          contém os nomes das imagens e as colunas subsequentes
                          contém os valores do vetor LBP.
    """
    lbp_vectors = []
    num_elementos_lbp = 0
    for nome_arquivo, img in imagens.items():
        # Converter para escala de cinza se a imagem for colorida
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        # Parâmetros para o LBP
        radius = 1
        n_points = 8 * radius
        # Calcular o LBP
        lbp = local_binary_pattern(gray, n_points, radius, method='uniform')
        # Calcular o histograma do LBP
        hist, _ = np.histogram(lbp.ravel(), bins=np.arange(0, n_points + 3), range=(0, n_points + 2))
        lbp_vectors.append([nome_arquivo] + hist.tolist())

    num_elementos_lbp = len(lbp_vectors[0]) - 1
    colunas = ["nome_arquivo"] + [f"LBP_{i}" for i in range(num_elementos_lbp)]
    df_lbp = pd.DataFrame(lbp_vectors, columns=colunas)
    print(f"Número de elementos do vetor LBP: {num_elementos_lbp}")
    return df_lbp



def calcula_RGB(imagens):
    """
    Calcula o histograma RGB para cada imagem.

    Args:
        imagens (dict): Um dicionário contendo os nomes dos arquivos
                       e as imagens carregadas.

    Returns:
        pandas.DataFrame: Um DataFrame onde a primeira coluna
                          contém os nomes das imagens e as colunas subsequentes
                          contêm os valores do histograma RGB.
    """
    rgb_vectors = []
    num_elementos_rgb = 0
    for nome_arquivo, img in imagens.items():
        hist_r = cv2.calcHist([img], [2], None, [256], [0, 256])
        hist_g = cv2.calcHist([img], [1], None, [256], [0, 256])
        hist_b = cv2.calcHist([img], [0], None, [256], [0, 256])
        rgb_vectors.append([nome_arquivo] + hist_r.flatten().tolist() + hist_g.flatten().tolist() + hist_b.flatten().tolist())

    num_elementos_rgb = len(rgb_vectors[0]) - 1
    colunas = ["nome_arquivo"] + [f"R_{i}" for i in range(256)] + [f"G_{i}" for i in range(256)] + [f"B_{i}" for i in range(256)]
    df_rgb = pd.DataFrame(rgb_vectors, columns=colunas)
    print(f"Número de elementos do vetor RGB: {num_elementos_rgb}")
    return df_rgb


#def salvar_features(pasta_imagens, df_hog, df_lbp, df_rgb):
def salvar_features(pasta_imagens, df_lbp, df_rgb):
    """
    Salva as features LBP e RGB em um arquivo de texto.

    Args:
        pasta_imagens (str): Caminho da pasta onde as imagens estão localizadas.
        #df_hog (pandas.DataFrame): DataFrame contendo as features HOG.
        df_lbp (pandas.DataFrame): DataFrame contendo as features LBP.
        df_rgb (pandas.DataFrame): DataFrame contendo as features RGB.
    """
    nome_arquivo_saida = os.path.join(pasta_imagens, "features_imagens_2024.csv")
    with open(nome_arquivo_saida, "w") as arquivo:
        # Escreve o cabeçalho
        #arquivo.write("nome_arquivo,HOG,LBP,RGB\n")
        arquivo.write("nome_arquivo,LBP,RGB\n")
        # Garante que os DataFrames tenham as mesmas imagens
        #merged_df = pd.merge(pd.merge(df_hog, df_lbp, on='nome_arquivo', how='inner'), df_rgb, on='nome_arquivo', how='inner')
        merged_df = pd.merge(df_lbp, df_rgb, on='nome_arquivo', how='inner')

        for _, row in merged_df.iterrows():
            nome_imagem = row['nome_arquivo']
            #hog_features = ','.join(map(str, row.filter(like='HOG_').values))
            lbp_features = ','.join(map(str, row.filter(like='LBP_').values))
            rgb_features = ','.join(map(str, row.filter(like='RGB').values)) + ',' + ','.join(map(str, row.filter(like='G_').values)) + ',' + ','.join(map(str, row.filter(like='B_').values))
            #arquivo.write(f"{nome_imagem},{hog_features},{lbp_features},{rgb_features}\n")
            arquivo.write(f"{nome_imagem},{lbp_features},{rgb_features}\n")
    print(f"Features salvas em: {nome_arquivo_saida}")



def ler_imagens(pasta_path):
    """
    Carrega todas as imagens JPG de uma pasta.

    Args:
        pasta_path (str): O caminho para a pasta contendo as imagens.

    Returns:
        dict: Um dicionário onde as chaves são os nomes dos arquivos
              e os valores são as imagens carregadas como arrays NumPy.
    """
    imagens = {}
    contador = 0
    for nome_arquivo in os.listdir(pasta_path):
        if nome_arquivo.lower().endswith(".jpg"):
            caminho_completo = os.path.join(pasta_path, nome_arquivo)
            img = cv2.imread(caminho_completo)
            if img is not None:
                imagens[nome_arquivo] = img
                contador += 1
            else:
                print(f"Erro ao ler a imagem: {nome_arquivo}")
    print(f"Número de imagens carregadas: {contador}")
    return imagens


if __name__ == '__main__':
    pasta_das_imagens = "./"  # Substitua pelo caminho real da sua pasta
    imagens_carregadas = ler_imagens(pasta_das_imagens)
    if imagens_carregadas:
        #df_hog = calcula_HOG(imagens_carregadas)
        df_lbp = calcula_LBP(imagens_carregadas)
        df_rgb = calcula_RGB(imagens_carregadas)
        #salvar_features(pasta_das_imagens, df_hog, df_lbp, df_rgb)
        salvar_features(pasta_das_imagens, df_lbp, df_rgb)
    else:
        print("Nenhuma imagem JPG encontrada na pasta.")
